#include "StdAfx.h"
#include "LabelledSetIntervalGenerator.h"
#include "Signature.h"
#include "resource.h"
#include "Interval.h"

using namespace Eyw;

////////////////////////////////////////////////////////////////////////////////////
// Signature
block_class_registrant g_LabelledSetIntervalGenerator( 
	block_class_registrant::block_id( "LabelledSetIntervalGenerator" )
		.begin_language( EYW_LANGUAGE_US_ENGLISH )
			.name( "LabelledSetIntervalGenerator" )
			.description( "A block which generates a labelled set of intervals"	)
			.libraries( "DataStructures.LabelledSet" )
			.bitmap( IDB_LABELLEDSETINTERVALGENERATOR_BITMAP )
		.end_language()	
		.begin_authors()
			.author( EYW_EYWWEEK2010_CATALOG_AUTHOR_ID )
		.end_authors()
		.begin_companies()
			.company( EYW_EYWWEEK2010_COMPANY_ID )
		.end_companies()
		.begin_licences()
			.licence( EYW_EYWWEEK2010_LICENSE_ID )
		.end_licences()
		.default_factory< CLabelledSetIntervalGenerator >()
	);

//////////////////////////////////////////////////////////////////////
// Identifiers
#define OUTPUT_OUTPUT "output"
#define PARAM_NUM_ITEMS "num_items"

/////////////////////////////////
CLabelledSetIntervalGenerator::CLabelledSetIntervalGenerator( const Eyw::OBJECT_CREATIONCTX* ctxPtr )
:	Eyw::CBlockImpl( ctxPtr )
{
	_pOutOutput=NULL;

}

CLabelledSetIntervalGenerator::~CLabelledSetIntervalGenerator()
{
}

void CLabelledSetIntervalGenerator::InitSignature()
{
	SetOutput(Eyw::pin::id(OUTPUT_OUTPUT)
	    .name("output")
	    .description("The generated labelled set of intervals")
	    .type<Eyw::ILabelledSet>()
	    );

	SetParameter(
		pin::id( PARAM_NUM_ITEMS )
			.name( "Number of items" )
			.description( "Maximum number of items added to the output labelled set" )
			.is_run_time(false)
			.set_int_domain()
				.min(0)
			);
	_pParNumItems = get_parameter_datatype<IInt>( PARAM_NUM_ITEMS );
	_pParNumItems->SetValue(1);

	OnChangedParameter( PARAM_NUM_ITEMS );


}

void CLabelledSetIntervalGenerator::CheckSignature()
{
	_signaturePtr->GetOutputs()->FindItem( OUTPUT_OUTPUT );
	_pParNumItems = get_parameter_datatype<IInt>( PARAM_NUM_ITEMS );
	CheckVariableParameters();
}

void CLabelledSetIntervalGenerator::DoneSignature()
{
	_pParNumItems = 0;
	_params.clear();
}

// Actions
bool CLabelledSetIntervalGenerator::Init() throw()
{
    try
    {
	    GetSchedulingInfo()->SetActivationTimeBased( true );
	    GetSchedulingInfo()->GetTimeBasedActivationInfo()->SetPeriodic( true );
	    GetSchedulingInfo()->GetTimeBasedActivationInfo()->SetPeriod( Eyw::TIME( 10000000.0 / 25 ) );

		_pOutOutput = get_output_datatype<Eyw::ILabelledSet>( OUTPUT_OUTPUT );
		labelled_set_init_info_ptr ptr = 
			datatype_init_info<Eyw::ILabelledSetInitInfo>::create( _kernelServicesPtr );
		ptr->SetCatalogID( "EywWeek2010" );
		ptr->SetClassID( "Interval" );
		_pOutOutput->InitInstance( ptr.get() );

    	return true;
    }
    catch(...)
    {
        return false;
    }
}

bool CLabelledSetIntervalGenerator::Start() throw()
{
    try
    {
    	// TODO: add your logic
    	return true;
    }
    catch(...)
    {
        return false;
    }
}

bool CLabelledSetIntervalGenerator::Execute() throw()
{
    try
    {
		_pOutOutput->Clear();
		int size = _params.size();
		interval_ptr ptr = datatype<IInterval>::create( _kernelServicesPtr/*, "EywWeek2010", "Interval"*/ );
		for( int i = 0; i < size; ++i )
		{
			std::string label = _params[i]._pLabel->GetValue();
		
			if( !label.empty() )
			{
				ptr->SetMin( _params[i]._pMin->GetValue() );
				ptr->SetMax( _params[i]._pMax->GetValue() );
				_pOutOutput->AddItem( ptr.get(), label.c_str() );
			}
		}
		_pOutOutput->SetCreationTime( _clockPtr->GetTime() );
    }
    catch(...)
    {
    }
	return true;
}

void CLabelledSetIntervalGenerator::Stop() throw()
{
    try
    {
    }
    catch(...)
    {
    }
}

void CLabelledSetIntervalGenerator::Done() throw()
{
    try
    {
		_pOutOutput = NULL;

    }
    catch(...)
    {
    }
}

void CLabelledSetIntervalGenerator::OnChangedParameter( const std::string& parameterId )
{
	if( parameterId == PARAM_NUM_ITEMS )
	{
		int old_num_items = _params.size();
		int num_items = _pParNumItems->GetValue();
		int i;
		for( i = 0; i < num_items; ++i )
		{
			std::string index = boost::lexical_cast<std::string>( i ) /* itoa */;
			SetParameter(
				pin::id( "label_" + index )
					.name( "Label " + index )
					.description( "Label of item " + index )
					.type<IString>()
					);
			SetParameter(
				pin::id( "min_" + index )
					.name( "Min " + index )
					.description( "Lower bound of interval " + index )
					.type<IDouble>()
					);
			SetParameter(
				pin::id( "max_" + index )
					.name( "Max " + index )
					.description( "Upper bound of interval " + index )
					.type<IDouble>()
					);
		}
		for( ; i < old_num_items; ++i )
		{
			std::string index = boost::lexical_cast<std::string>( i ) /* itoa */;
			std::string id = "label_"+index;
			std::string min_id = "min_"+index;
			std::string max_id = "max_"+index;
			RemoveParameter( id );
			RemoveParameter( min_id );
			RemoveParameter( max_id );
		}
		CheckVariableParameters();
		Notify_SignatureChanged();
	}

	/* This is to enumerate all Eyw datatypes
	nsc::string_pair_set interface_ids;
	const_datatype_class_ptr_vector result = get_datatypes_implementing_interfaces( _kernelServicesPtr, interface_ids );
	*/
}

void CLabelledSetIntervalGenerator::CheckVariableParameters()
{
	int num_items = _pParNumItems->GetValue();
	_params.resize( num_items );
	for( int i = 0; i < num_items; ++i )
	{
		std::string index = boost::lexical_cast<std::string>( i ) /* itoa */;
		_params[ i ]._pLabel = get_parameter_datatype<IString>("label_" + index);
		_params[ i ]._pMin = get_parameter_datatype<IDouble>("min_" + index);
		_params[ i ]._pMax = get_parameter_datatype<IDouble>("max_" + index);
	}
}